#!/usr/bin/env python

def intarray_to_c(ary, name):
    parts = []
    parts.append("static const size_t {0}[{1}] = {{".format(name,len(ary)))
    count = 0
    for val in ary:
        if (count % 10 == 0):
            parts.append("\n  ")
        parts.append("{0}, ".format(val))
        count += 1
    parts.append("\n};\n")
    return ''.join(parts)

def chararray_to_c(ary, name):
    parts = []
    parts.append("static const uint8_t {0}[{1}] = {{".format(name,
                                                             len(ary)))
    count = 0;
    for val in ary:
        if (count % 10 == 0):
            parts.append("\n ")
        parts.append("{0:>6}".format("'" + val + "',"))
        count += 1

    parts.append("\n};\n")
    return ''.join(parts)

def json_encodelen_map():
    lenmap = [ 6 for i in range(256) ]
    for i in range(32,128):
        lenmap[i] = 1
    for i in [0x08, 0x09, 0x0a,  0x0c, 0x0d, 0x5c, 0x22]:
        lenmap[i] = 2
    return lenmap

def json_encode_map():
    encodemap = [ 'u' for i in range(256) ]

    # things to quad-hex encode  \u00XY
    for i in range(32,128):
        encodemap[i] = 'a';

    # special escapes
    encodemap[0x08] = 'b'
    encodemap[0x09] = 't'
    encodemap[0x0a] = 'n'
    encodemap[0x0c] = 'f'
    encodemap[0x0d] = 'r'
    encodemap[0x5c] = "\\\\"
    encodemap[0x22] = "\""

    return encodemap

if __name__ == '__main__':

    with open('modp_json_data.h', 'w') as fd:
        fd.write(chararray_to_c(json_encode_map(), "gsJSONEncodeMap"))
        fd.write("\n")
        fd.write(intarray_to_c(json_encodelen_map(), "gsJSONEncodeLenMap"))



#!/usr/bin/env python


class Trie(object):
    def __init__(self):
        self.struct = [
            [ ]
        ]

    def dump(self):
        count = 0
        for i in self.struct:
            count += len(i)
            print i
        print len(self.struct)
        print count

    def longest_match(self, s):
        """
        sample code
        """
        leaf = self.struct[0]
        matches = []
        for ch in s:
            newleaf = None
            for node in leaf:
                if ch == node['value']:
                    found = True
                    if node['userdata'] is not None:
                        matches.append(node['userdata'])
                    if node['next'] == 0:
                        return matches
                    newleaf = self.struct[node['next']]
                    break
            if newleaf:
                leaf = newleaf
            else:
                break
        return matches

    def compile(self):
        """
        The current data struct is a a ragged array
        0 - [ ]
        1 - [   ]
        2 - [      ]
        3 - [  ]
        i.e. an array of vectors which are ugly to
        statically allocate in C
        http://stackoverflow.com/questions/1083658/jagged-array-in-c

        value, more, next, [userdata or 0]
        """
        offsets = [ 0 ]
        count = 0
        for leaf in self.struct:
            count += len(leaf)
            offsets.append(count)
        print """
#ifndef _MODP_HTML_NAMED_ENTITIES_H
#define _MODP_HTML_NAMED_ENTITIES_H
#include <string.h>

typedef struct trienode {
    char value;    /* 1 byte -- char value - we deal only in pure ASCII */
    char more;     /* 1 byte -- should be read next value in array (boundary marker) */
    short next;    /* 2 byte -- position of next leaf */
    int codepoint; /* 4 byte -- codepoint or 0 if none */
} trienode_t;

static const trienode_t entities[];
static int html_named_entity_decode(const char* s, size_t len, size_t* consumed);

static int html_named_entity_decode(const char* s, size_t len, size_t* consumed)
{
   int codepoint = 0;
   size_t pos = 0;
   int i = 0;

   *consumed = 0;

   while (pos < len) {
        if (s[pos] == entities[i].value) {
           if (entities[i].codepoint) {
               codepoint = entities[i].codepoint;
               *consumed = pos + 1;
           }
           if (entities[i].next != 0) {
               i = entities[i].next;
               pos += 1;
               continue;
           } else {
               break;
           }
        }
        if (entities[i].more == 0) {
           break;
        }
        ++i;
   }
   return codepoint;
}
/*
#include <stdio.h>
int main(int argc, char* argv[]) {
    size_t consumed;
    int codepoint;
    codepoint = html_named_entity_decode(argv[1], strlen(argv[1]), &consumed);
    printf("cp = %d, consumed = %d\\n", codepoint, consumed);
    return 0;
}
*/
static const trienode_t entities[] = {
"""
        for leaf in self.struct:
            nodelen = 0
            for node in leaf:
                nodelen += 1
                if node['full'] != None:
                    comment = ' /* &' + node['full'] + ' */'
                else:
                    comment = ''

                print "{{ {0}, {1}, {2}, {3} }},{4}".format(ord(node['value']),
                                                            int(nodelen != len(leaf)),
                                                            offsets[node['next']],
                                                            node['userdata'],
                                                            comment
                                                        )
        print "};"
        print "#endif"

    def add(self, s, userdata):
        idx = 0
        leaf = self.struct[0]
        count = 0
        for ch in s[0:-1]:
            #print "CHAR " + i
            key = None
            for node in leaf:
                if ch == node['value']:
                    key = node
                    break
            if key is None:
                #print "key is none"
                newleaf = [ ]
                self.struct.append(newleaf)
                idx = len(self.struct) -1
                leaf.append( {
                    "full": None,
                    "value": ch,
                    "userdata": 0,
                    "next": idx
                } )
                leaf = newleaf
            elif key['next'] != 0:
                # just follow
                #print "got key index of " + str(key[1])
                leaf = self.struct[key['next']]
            else:
                newleaf = [ ]
                self.struct.append(newleaf)
                idx = len(self.struct) -1
                key['next'] = idx
                leaf = newleaf

        key = None
        ch = s[-1]
        for node in leaf:
            if ch == node['value']:
                key = node
                break
        if key is None:
            leaf.append( {
                'full': s,
                'userdata': userdata,
                'next': 0,
                'value': s[-1]
            })
        else:
            key['full'] = s
            key['userdata'] = userdata



import json
with open('entities.json', 'r') as fd:
    obj = json.load(fd)

t = Trie()
total = 0
for name, data in obj.iteritems():
    total += len(name) -1
    t.add(name[1:], data['codepoints'][0])

#for i in ["fo", "fa", "foo"]:
#    t.add(i)

#t.dump()
#print t.longest_match("amp;")

t.compile()




#!/usr/bin/env python

#
# This is the MIT License
# http://www.opensource.org/licenses/mit-license.php
#
# Copyright (c) 2008 Nick Galbreath
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#

gsCharToInt = [
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256,   0, 256,   1,
    2,   3, 256,   4,   5,   6,   7,   8, 256,   9,  10,  11,
    12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22, 256,
    23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
    35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
    47,  48,  49,  50,  51,  52,  53,  54, 256,  55,  56,  57,
    58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
    70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
    82,  83,  84, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256, 256,
    256, 256, 256, 256]

gsIntToChar = [
    '!',  '#',  '$',  '%', '\'',  '(',  ')',  '*',  '+',  '-',
    '.',  '/',  '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',
    '8',  '9',  ':',  '<',  '=',  '>',  '?',  '@',  'A',  'B',
    'C',  'D',  'E',  'F',  'G',  'H',  'I',  'J',  'K',  'L',
    'M',  'N',  'O',  'P',  'Q',  'R',  'S',  'T',  'U',  'V',
    'W',  'X',  'Y',  'Z',  '[',  ']',  '^',  '_',  '`',  'a',
    'b',  'c',  'd',  'e',  'f',  'g',  'h',  'i',  'j',  'k',
    'l',  'm',  'n',  'o',  'p',  'q',  'r',  's',  't',  'u',
    'v',  'w',  'x',  'y',  'z'
    ]

from struct import unpack, pack
# covert 4 characters into 5
def b85_encode(s):
    parts = []
    numchunks = len(s) // 4
    format = '!' + str(numchunks) + 'I'
    for x in  unpack(format, s):
        # network order (big endian), 32-bit unsigned integer
        # note: x86 is little endian
        parts.append(gsIntToChar[x // 52200625])
        parts.append(gsIntToChar[(x // 614125) % 85])
        parts.append(gsIntToChar[(x // 7225) % 85])
        parts.append(gsIntToChar[(x // 85) % 85])
        parts.append(gsIntToChar[x % 85])
    return ''.join(parts)

#
# also does not use the 'struct' module which may be desirable
# to some
#
def b85_encode2(s):
    parts = []
    for i in xrange(0,len(s), 4):
        chunk = s[i:i+4]
        x = ord(chunk[3]) + 256*(ord(chunk[2]) + 256*(ord(chunk[1]) + 256*ord(chunk[0])))

        # network order (big endian), 32-bit unsigned integer
        # note: x86 is little endian
        parts.append(gsIntToChar[x // 52200625])
        parts.append(gsIntToChar[(x // 614125) % 85])
        parts.append(gsIntToChar[(x // 7225) % 85])
        parts.append(gsIntToChar[(x // 85) % 85])
        parts.append(gsIntToChar[x % 85])
    return ''.join(parts)

# convert 5 characters to 4
def b85_decode(s):
    parts = []
    for i in xrange(0, len(s), 5):
        bsum = 0;
        for j in xrange(0,5):
            val = gsCharToInt[ord(s[i+j])]
            bsum = 85*bsum + val
        tmp = pack('!I', bsum)
        parts.append(tmp)
        #parts += tmp
        #parts += unpack('cccc', tmp)
    return ''.join(parts)

# convert 5 characters to 4
def b85_decode2(s):
    parts = []
    for i in xrange(0, len(s), 5):
        bsum = 0;
        for j in xrange(0,5):
            val = gsCharToInt[ord(s[i+j])]
            bsum = 85*bsum + val
        parts.append(chr((bsum >> 24) & 0xff))
        parts.append(chr((bsum >> 16) & 0xff))
        parts.append(chr((bsum >> 8) & 0xff))
        parts.append(chr(bsum & 0xff))

    return ''.join(parts)

import unittest
class B85Test(unittest.TestCase):

    def testDecode1(self):
        s = b85_decode('!!!!#')
        self.assertEquals(4, len(s))
        self.assertEquals(0, ord(s[0]))
        self.assertEquals(0, ord(s[1]))
        self.assertEquals(0, ord(s[2]))
        self.assertEquals(1, ord(s[3]))

        e = b85_encode(s)
        self.assertEquals('!!!!#', e)


if __name__ == '__main__':
    from time import clock
    #unittest.main()

    s = '!!!!#' * 10

    t0 = clock()
    for i in xrange(1000000):
        b85_decode(s)
    t1 = clock()
    print "decode v1",  t1-t0

    t0 = clock()
    for i in xrange(1000000):
       b85_decode2(s)
    t1 = clock()
    print "decode v2",  t1-t0


    s = b85_decode('!!!!#' * 10)

    t0 = clock()
    for i in xrange(1000000):
       b85_encode(s)
    t1 = clock()
    print "encode v1",  t1-t0

    t0 = clock()
    for i in xrange(1000000):
        b85_encode2(s)
    t1 = clock()
    print "encode v2",  t1-t0


